/***************************************************************************
 *   Copyright (C) 2005 by Mark Six                                        *
 *   marksix@xs4all.nl                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "codeeditor.h"
#include <klibloader.h>

#include <qmessagebox.h>
#include <kmessagebox.h>
#include <qlayout.h>
#include <kfiledialog.h>

CodeEditor::CodeEditor(QWidget *parent, const char *name)
 : QWidget(parent, name)
{
	QVBoxLayout *layout = new QVBoxLayout(this) ;
		
	KLibFactory *factory = KLibLoader::self()->factory( "libkatepart" ) ;
	m_doc =  ( Kate::Document* ) factory->create( 0L, "kate", "KTextEditor::Document" ) ;
	m_view = ( Kate::View * ) m_doc->createView( this ) ;
	m_view->setLineNumbersOn( true ) ;
	m_view->setIconBorder( true ) ;
	
	layout->addWidget( m_view ) ;
	
	m_exeLine = 0 ;
	m_filename = "" ;
	m_bFilename = false ;
	m_doc->setModified( false );
	
	setHighlightMode() ;

}

CodeEditor::~CodeEditor()
{
	if ( m_view->document()->views().count() == 1) 
		delete m_view->document();
}

void CodeEditor::slotToggleBreakpoint()
{
	unsigned int line, col ;
	m_view->cursorPosition( &line, &col ) ;
	
	QPtrList<KTextEditor::Mark> markList = m_doc->marks() ;
	
	unsigned int i ;
	for ( i = 0 ; i < markList.count() ; i++ )
		if ( markList.at( i )->line == line && markList.at( i )->type == KTextEditor::MarkInterface::markType02 )
			break ;
	
	if ( i == markList.count() )	
		m_doc->setMark( line,  KTextEditor::MarkInterface::markType02 ) ; 
	else
		m_doc->removeMark( line, KTextEditor::MarkInterface::markType02 ) ;
}

void CodeEditor::clearExecutionMarker()
{
	m_doc->removeMark( m_exeLine, KTextEditor::MarkInterface::markType05 ) ;
}

void CodeEditor::setCursor( unsigned int line )
{
	m_view->setCursorPosition( line, 0 ) ;
}

void CodeEditor::setExecutionMarker( unsigned int line )
{
	m_doc->removeMark( m_exeLine, KTextEditor::MarkInterface::markType05 ) ;
	m_exeLine = line ;
	if ( isBreakpoint( line ) ) {
		m_doc->setMark( m_exeLine, KTextEditor::MarkInterface::markType05 | KTextEditor::MarkInterface::markType02 ) ;
	} else
		m_doc->setMark( m_exeLine, KTextEditor::MarkInterface::markType05 ) ;

	setCursor( line ) ;	// KDevelop does this too!

}

bool CodeEditor::isBreakpoint( unsigned int line )
{
	return ( ( m_doc->mark( line ) &  KTextEditor::MarkInterface::markType02 ) != 0 ) ;
}

void CodeEditor::slotNewFile()
{
	m_doc->clear() ;
	m_doc->setModified( false ) ;
	m_filename = "" ;
	m_bFilename = false ;
}

void CodeEditor::slotSaveAs()
{
	if ( m_view->saveAs() == Kate::View::SAVE_OK ) {
		m_filename = m_doc->url().path() ;
		m_bFilename = true ;
	}
}

bool CodeEditor::save()
{
	slotSave() ;
	return m_bFilename ;
}

void CodeEditor::slotSave()
{
	if ( !m_bFilename )
		slotSaveAs() ;
	else if ( m_view->save() == Kate::View::SAVE_OK ) {
		m_filename = m_doc->url().path() ;
		m_bFilename = true ;
	}
}

void CodeEditor::slotPrint()
{
	m_doc->print() ;
}

void CodeEditor::slotFind()
{
	m_view->find() ;
}

void CodeEditor::slotFindNext()
{
	m_view->findAgain() ;
}

void CodeEditor::slotShowConfig()
{
	m_doc->configDialog() ;
}

void CodeEditor::slotCut()
{
	m_view->cut() ;
}

void CodeEditor::slotCopy()
{
	m_view->copy() ;
}

void CodeEditor::slotPaste()
{
	m_view->paste() ;
}

void CodeEditor::slotSelectAll()
{
	m_doc->selectAll() ;
}	

void CodeEditor::slotUndo()
{
	m_doc->undo() ;
}

void CodeEditor::slotRedo()
{
	m_doc->redo() ;
}

void CodeEditor::slotOpen()
{
	QString filename = KFileDialog::getOpenFileName( QString::null, QString( "*.psm|PicoBlaze assembly files" ) ) ;
	if ( filename != "" ) {
		m_filename = filename ;
		m_bFilename = true ;
		m_doc->openURL( filename ) ;
		setHighlightMode() ;
	}
}

QString CodeEditor::getFilename()
{
	return m_filename ;
}

	
void CodeEditor::setHighlightMode()
{		
	int i = 0;
	int hlModeCount = m_doc->hlModeCount();
	while ( i < hlModeCount ) {
		if ( m_doc->hlModeName( i ) == "pblazeASM" ) {
			m_doc->setHlMode( i ) ;
			break ;
		}
		i++;
	}
}

bool CodeEditor::askSave()
{
	QString filename ;
	if ( m_bFilename )
		filename = m_filename ;
	else
		filename = "Untitled" ;

	int choice = KMessageBox::warningYesNoCancel( this, QString( "The document \'%1\' has been modified.\nDo you want to save it?").arg( filename ), "Save Document?", KGuiItem( "Save" ), KGuiItem( "Discard" ) );

    if ( choice == KMessageBox::Yes ) 
		save() ;

	return ( choice != KMessageBox::Cancel ) ;
}

bool CodeEditor::close()
{
	if ( m_doc->isModified() ) 
		return askSave() ;
}




#include "codeeditor.moc"
